@@ -15,6 +15,33 @@ from nektar import Waggle
 
 KNOWN_WORDS = STOP_WORDS+ENGLISH_WORDS
 
+RE_DASH = re.compile(r"-")
+RE_EN_DASH = re.compile(r"\u2013")
+RE_EM_DASH = re.compile(r"\u2014")
+RE_N_RANK = re.compile(r"\#[\d]+")
+RE_DOLLARS = re.compile(r"\$[\d\,\.]+")
+RE_POSSESSIVE = re.compile(r"\b[\w]+[']s\b")
+RE_MULTI_SPACE = re.compile(r"\s{2,}")
+RE_CLEAN_TITLE = re.compile(r"[^\w\'\,\ ]+")
+RE_DELIMITERS = re.compile(r"[\n\.]")
+RE_IMAGES = re.compile(r"!\[[\w\"\-\.~!#$%&'()*+,;=:@\/?\ ]*\]\([\w\"\-\.~!#$%&'()*+,;=:@\/?\ ]+\)")
+RE_HIVE_SVC = re.compile(r"\[\/\/\]:#[\ ]+\([!][\w\ \.]+\)")
+RE_ASTERISKS = re.compile(r"[\*]+")
+RE_TILDES = re.compile(r"[~]+")
+RE_UNDERSCORES = re.compile(r"[_]+")
+RE_HEADERS = re.compile(r"[#]+")
+RE_CODE_BLOCKS = re.compile(r"[`]+[\w]*[\ ]*")
+RE_TABLE_SEP = re.compile(r"\|[\-]+\|[\-]+\|")
+RE_PIPES = re.compile(r"[\ ]*\|[\ ]*")
+RE_HTML_TAGS = re.compile(r"<[\/]?[a-zA-Z]+[1-6]?[\w\"\-\.~!#$%&'()*+,;=:@\/?\ ]+>")
+RE_LINKS_RIGHT = re.compile(r"\]\([\w\"\-\.~!#$%&'*+,;=:@\/?\ ]+\)")
+RE_BLOCKQUOTES = re.compile(r">[\ ]?")
+RE_HR = re.compile(r"--[\-]+")
+RE_TRAILING_PARENTHESIS = re.compile(r"[\(\[\{\}\]\)]")
+RE_USER_TAGS = re.compile(r"@[a-z0-9\-\.]{3,16}")
+RE_NON_ASCII = re.compile(r"[^ -~]")
+RE_WORDS = re.compile(r"\b\w\w+\b")
+
 class Scrutineer:
     def __init__(
         self, minimum_score=80, max_emojis=0, max_user_tags=5, max_tags=5, retries=1, deep=False
@@ -38,7 +65,7 @@ class Scrutineer:
             float(tags),
         ]
 
-    def analyze(self, post, permlink=None, skip_bad_title=True):
+    def analyze(self, post, permlink=None, auto_skip=False):
         author = post
         if isinstance(post, dict):
             author = post["author"]
@@ -54,7 +81,7 @@ class Scrutineer:
         if not len(title):
             return {}
         self.analysis["title"] = _analyze_title(title)
-        if self.analysis["title"]["readability"] < (self._minimum_score / 100):
+        if auto_skip and (self.analysis["title"]["readability"] < 0.8):
             return {}
 
         body = post["body"]
@@ -76,13 +103,17 @@ class Scrutineer:
         if not len(cleaned):
             return {}
         
-        self.analysis["body"] = _analyze_body(cleaned, self._deep)
+        self.analysis["body"] = {}
         self.analysis["emojis"] = _analyze_emojis(body, self._max_emojis)
+        if auto_skip and (self.analysis["emojis"]["score"] < 0.8):
+            return {}
+        
+        self.analysis["body"] = _analyze_body(cleaned, self._deep)
 
         if self._deep:
             keywords = self.analysis["body"]["seo_keywords"]
             self.analysis["title"] = _analyze_title(title, keywords)
-        if self.analysis["title"]["readability"] < (self._minimum_score / 100):
+        if auto_skip and (self.analysis["title"]["readability"] < 0.8):
             return {}
 
         word_count = self.analysis["body"]["cleaned"]
@@ -200,17 +231,17 @@ def _analyze_title(title, keywords=None):
     analysis = {}
     # analysis["title"] = title
     
-    title = re.sub(r"-", " ", title)
-    title = re.sub(r"\u2013", " ", title)
-    title = re.sub(r"\u2014", " ", title)
-    title = re.sub(r"\#[\d]+", " ", title)
-    title = re.sub(r"\$[\d\,\.]+", " ", title)
-    title = re.sub(r"\b[\w]+[']s\b", "", title)
-    title = re.sub(r"\s{2,}", " ", title).strip()
+    title = RE_DASH.sub(" ", title)
+    title = RE_EN_DASH.sub(" ", title)
+    title = RE_EM_DASH.sub(" ", title)
+    title = RE_N_RANK.sub(" ", title)
+    title = RE_DOLLARS.sub(" ", title)
+    title = RE_POSSESSIVE.sub("", title)
+    title = RE_MULTI_SPACE.sub(" ", title).strip()
     length = len(title.encode("utf-8"))
     analysis["title"] = title
     
-    words = re.sub(r"[^\w\'\,\ ]+", " ", title.lower()).split(" ")
+    words = RE_CLEAN_TITLE.sub(" ", title.lower()).split(" ")
     cleaned = " ".join([w for w in words if w in KNOWN_WORDS])
     analysis["cleaned"] = cleaned.strip()
     
@@ -235,35 +266,29 @@ def _analyze_title(title, keywords=None):
 
 def _parse_body(body):
     # remove images, replace whitespaces
-    pattern = (
-        r"!\[[\w\"\-\.~!#$%&'()*+,;=:@\/?\ ]*\]\([\w\"\-\.~!#$%&'()*+,;=:@\/?\ ]+\)"
-    )
-    cleaned = re.sub(pattern, "", body).lower()
-    cleaned = re.sub(r"[\n\.]", " ", cleaned)
-    cleaned = re.sub(r"\u2013", " ", cleaned)
-    cleaned = re.sub(r"\u2014", " ", cleaned)
+    cleaned = RE_IMAGES.sub("", body).lower()
+    cleaned = RE_DELIMITERS.sub(" ", cleaned)
+    cleaned = RE_EN_DASH.sub(" ", cleaned)
+    cleaned = RE_EM_DASH.sub(" ", cleaned)
 
     ## remove other formatting codes
     ## do not change order of patterns !!
-    patterns = [
-        r"\[\/\/\]:#[\ ]+\([!][\w\ \.]+\)" r"#+",  # hive services codes
-        r"[*]+",
-        r"[~]+",
-        r"[_]+",
-        r"[#]+",  # code blocks
-        r"[`]+[\w]*[\ ]?",  # code blocks
-        r"\|[\-]+\|[\-]+\|",  # table separator
-        r"[\ ]*\|[\ ]*",  # remove pipe symbols
-        r"<[\/]?[a-zA-Z]+[1-6]?[\w\"\-\.~!#$%&'()*+,;=:@\/?\ ]+>",  # html tags
-        r"\]\([\w\"\-\.~!#$%&'*+,;=:@\/?\ ]+\)",  # links, right-side
-        r">[\ ]?",
-        r"--[\-]+",  # blocks, horizontal rule
-        r"[\(\[\{\}\]\)]",  # trailing parentheses, brackets, and braces
-        r"@[a-z0-9\-\.]{3,16}",  # user tags
-    ]  
-    for pattern in patterns:
-        cleaned = re.sub(pattern, "", cleaned)
-    cleaned = re.sub(r"\s{2,}", " ", cleaned)
+    cleaned = RE_HIVE_SVC.sub("", cleaned)
+    cleaned = RE_ASTERISKS.sub("", cleaned)
+    cleaned = RE_TILDES.sub("", cleaned)
+    cleaned = RE_UNDERSCORES.sub("", cleaned)
+    cleaned = RE_HEADERS.sub("", cleaned)
+    cleaned = RE_CODE_BLOCKS.sub("", cleaned)
+    cleaned = RE_TABLE_SEP.sub("", cleaned)
+    cleaned = RE_PIPES.sub("", cleaned)
+    cleaned = RE_HTML_TAGS.sub("", cleaned)
+    cleaned = RE_LINKS_RIGHT.sub("", cleaned)
+    cleaned = RE_BLOCKQUOTES.sub("", cleaned)
+    cleaned = RE_HR.sub("", cleaned)
+    cleaned = RE_TRAILING_PARENTHESIS.sub("", cleaned)
+    cleaned = RE_USER_TAGS.sub("", cleaned)
+    cleaned = RE_MULTI_SPACE.sub(" ", cleaned)
+    
     return cleaned
 
 def _analyze_body(cleaned, deep):
@@ -284,7 +309,8 @@ def _analyze_body(cleaned, deep):
 def _analyze_emojis(body, limit=0):
     analysis = {}
     analysis["max_emojis"] = limit
-    analysis["emojis"] = [c for c in body if c in EMOJIS]
+    chars = list(RE_NON_ASCII.findall(body))
+    analysis["emojis"] = [c for c in chars if c in EMOJIS]
     analysis["count"] = len(analysis["emojis"])
     analysis["score"] = 1
     if analysis["count"] > int(limit):
@@ -297,7 +323,7 @@ def _get_bigrams(contents, occurrence=4, limit=5):
     limit = int(limit)
 
     bigrams = {}
-    words = list(re.findall(r"\b\w\w+\b", contents.lower()))
+    words = list(RE_WORDS.findall(contents.lower()))
     words = [x for x in words if x not in STOP_WORDS]
     for i in range(len(words) - 2):
         bigram = " ".join(words[i : i + 2])
@@ -314,8 +340,7 @@ def _get_bigrams(contents, occurrence=4, limit=5):
 def _analyze_images(body, word_count):
     analysis = {}
     ## get image to text ratio
-    pattern = r"!\[[\w\ \-\._~!$&'()*+,;=:@#\/?]*\]\([\w\-\.~!$%&'()*+,;=:@\/?]+\)"
-    analysis["count"] = len(list(re.findall(pattern, body)))
+    analysis["count"] = len(list(RE_IMAGES.findall(body)))
     analysis["score"] = 0
     if analysis["count"]:
         scores = [0]
@@ -330,7 +355,7 @@ def _analyze_images(body, word_count):
 
 def _analyze_overtagging(body, max_user_tags):
     analysis = {}
-    tags = len(re.findall(r"@[a-z0-9\-\.]{3,16}", body))
+    tags = len(RE_USER_TAGS.findall(body))
     analysis["max_user_tags"] = max_user_tags
     analysis["count"] = tags
     analysis["score"] = 1
